import time
import baco
from threading import *
from AudioSource import *
from synapps import SynAppsClient, SynAppsConfiguration, player_iface


class SynAppsAudioSource(AudioSource):

    def __init__(self, sourceId, sourceName, sourceVolume, sourceConfig, audio_dev):
        AudioSource.__init__(self, sourceId, sourceName, sourceVolume, sourceConfig, audio_dev)

        # Audio Source configuration variables
        self.playing = False
        self.externalCallback = None
        self.last_command = {}
        self.volume = sourceVolume

        self.logger = logging.getLogger('ipspeaker')
        self.running = False
        self.lock = Lock()

        conf = audio_dev.split(':')
        self.audioChannel = conf[1]
        # Baco Lib Player
        self.player = None
        self.monitoringThread = None
        self.isMonitoring = False

        # max number of streams
        self.LIST_SIZE = 10

        # list to store streams received ordered by priority (max 10)
        self.streamsList = [None] * self.LIST_SIZE
        # stream data that is currently playing
        self.currently_playing = None
        # list to store monitoring streams objects (max 10)
        self.monitoringList = [None] * self.LIST_SIZE

        # number of times SynappsAudioSource was told to play
        self.counter = 0

        # registration result
        self.regResult = False

        # synapps client configuration
        self.synapps = self.initSynAppsClient()

        # start synapps client
        self.start()

    def initSynAppsClient(self):
        # get Server Address Configuration
        param = AudioSource.getParameter(self, "server")
        if param == "":
            serverconfig = "auto"
        else:
            serverconfig = param

        synappsConfig = SynAppsConfiguration.SynAppsConfig(synappsServerAddr=serverconfig, CommandProtocol="TCP")

        # create SynAppsClient
        synappsClient = SynAppsClient.SynAppsDevice(synappsConfig)

        # insert callback functions in handler
        synappsClient.createHandler(self)

        """
        #synapps client always starts in NONE_PLAYING_STATUS: need to specify when it is going to use custom handler
        self.synapps.setPlayerStatus(self.synapps.NONE_PLAYING_STATUS)
        """

        return synappsClient

    def start(self):
        self.lock.acquire()
        try:
            if self.running == False:
                # initialize Thread for receiving Commands
                self.synapps.connect()
                self.running = True
        finally:
            # register synapps client in Synapps server; callback to retrieve the registration result
            self.synapps.registerDevice(self.registrationResultCallback)
            self.lock.release()

    def onPaStart(self, obj):
        # print("------------- onPaStart ------------")
        if self.playing:
            # if self.synapps.getPlayerStatus() != self.synapps.NONE_PLAYING_STATUS:
            # if it is playing a stream with lowest priority
            if self.currently_playing is not None and self.currently_playing.priority >= obj.priority:
                # stop stream that is playing
                self.isMonitoring = False
                self.player.disconnect()

                self.logger.info("SYN_APPS_CLIENT: stopped playing stream - rtp://{}:{}".format(self.currently_playing.stream_ip, self.currently_playing.port))
                self.logger.info("SYN_APPS_CLIENT: will play stream - rtp://{}:{}".format(obj.stream_ip, obj.port))
                # play new stream
                self.startStream(obj)
            elif self.currently_playing is not None and self.currently_playing.priority < obj.priority:
                # if it is playing a stream with higher priority
                self.logger.info(
                    "SYN_APPS_CLIENT: won't play stream - rtp://{}:{} because a stream with higher priority is playing - rtp://{}:{}".format(
                        obj.stream_ip, obj.port, self.currently_playing.stream_ip, self.currently_playing.port))
            else:
                # play new stream
                self.logger.info("SYN_APPS_CLIENT: will play stream - rtp://{}:{}, currently_playing is None".format(obj.stream_ip, obj.port))
                self.startStream(obj)
        else:
            self.logger.info("SYN_APPS_CLIENT: will play stream - rtp://{}:{}, state is not in PLAYING".format(obj.stream_ip,obj.port))
            self.startStream(obj)

        # print("------------- end onPaStart ------------")
        return self.synapps.SUCCESS_MSG

    def onPaStop(self, obj):
        # print("------------- onPaStop ------------")
        # self.logger.debug("cmd.stream_ip: {}, cmd.port: {}, cmd.port.type: {}, self.announce_obj.stream_ip: {}, self.announce_mon_port: {}".format(cmd.stream_ip, cmd.port, type(cmd.port), self.announce_obj.stream_ip, self.announce_mon_port))
        self.logger.info("SYN_APPS_CLIENT: received command stop stream - rtp://{}:{}".format(obj.stream_ip, obj.port))

        if self.playing:
            if obj.stream_ip == "0.0.0.0" and int(obj.port) == 0:
                self.isMonitoring = False
                self.player.disconnect()
                self.logger.info(
                    "SYN_APPS_CLIENT: stopped playing stream - rtp://{}:{}".format(self.currently_playing.stream_ip,
                                                                                   self.currently_playing.port))
                self.currently_playing = None

            else:
                # if it corresponds to the stream that is playing, stop it
                if obj.stream_ip == self.currently_playing.stream_ip and obj.port == self.currently_playing.port:
                    self.isMonitoring = False
                    self.player.disconnect()
                    if self.externalCallback is not None:
                        self.externalCallback(self, False)
                    self.logger.info("SYN_APPS_CLIENT: stopped playing stream - rtp://{}:{}".format(
                        self.currently_playing.stream_ip, self.currently_playing.port))
                    self.currently_playing = None

        else: # elif self.state == SourceState.MONITORING:
            self.logger.info(
                "SYN_APPS_CLIENT: will stop monitoring stream - rtp://{}:{}".format(obj.stream_ip, obj.port))
            self.isMonitoring = False
            #self.player.disconnect()
            # print("------------- end onPaStop ------------")

        return self.synapps.SUCCESS_MSG

    def registrationResultCallback(self, result):
        self.regResult = result

    # adjusts volume, start monitoring, start to play and start to generate statistics
    def startStream(self, obj):
        self.setVolume(obj.volume)
        self.logger.info("SYN_APPS_CLIENT: will play stream - rtp://{}:{}".format(obj.stream_ip, obj.port))
        url = "rtp://{}:{}".format(obj.stream_ip, obj.port)
        self.player = baco.SimplePlayer(url, AudioSource.getAudioDev(self), 44100)
        self.logger.info("SYN_APPS_CLIENT: Created SimplePlayer ")
        self.player.setInputFifoDuration(0)
        self.player.setOutputFifoDuration(0)
        self.player.setCallback(self.checkStateCallback)
        self.player.connectSource()
        self.currently_playing = obj
        self.startMonitoring()

    def setCallback(self, callback):
        self.externalCallback = callback

    def checkStateCallback(self,alive):
        if alive:
            self.logger.info("SYN_APPS_CLIENT: stream is ALIVE")
        else:
            self.logger.info("SYN_APPS_CLIENT: stream is DEAD")
        if self.externalCallback is not None:
            self.externalCallback(self, alive)

    def startMonitoring(self):
        if not self.isMonitoring:
            self.isMonitoring = True
            self.monitoringThread = Thread(target=self.reconnectSourceLoop)
            self.monitoringThread.start()

    def reconnectSourceLoop(self):
        while self.isMonitoring:
            #self.logger.info("Synapps - reconnectSourceLoop")
            isConnected = self.player.isConnected()
            if not isConnected:
                res = self.player.connectSource()
                if res == 0:
                    self.logger.info("Synapps source id={}, stream connection succeed!".format(self.id))
                    if self.playing:
                        self.player.connectAudio()
                else:
                    self.logger.info("Synapps source id={}, stream connection failed.".format(self.id))

            time.sleep(1)
        self.isMonitoring = False

    def stopMonitoring(self, idx):
        if self.monitoringList[idx] is not None:
            self.monitoringList[idx].disconnect()
            self.monitoringList[idx] = None

    def connect(self):
        pass

    def disconnect(self):
        self.stopAudio()

    def shutdown(self):
        with self.lock:
            self.logger.info(f"Shutdown source id={self.id}, name={self.name}")
            try:
                if self.running == True:
                    self.playing = False
                    self.isMonitoring = False
                    if self.player is not None:
                        if self.player.isConnected():
                            self.player.disconnect()
                    self.synapps.disconnect()
                    self.running = False
            except Exception as e:
                self.logger.error(f"Shutdown Exception: {e}")

    def playAudio(self):
        with self.lock:
            self.logger.info(f"Playing source id={self.id}, name={self.name}")
            try:
                self.logger.info("Will connect audio!")
                self.player.connectAudio()
                self.playing = True
            except Exception as e:
                self.logger.error(f"Play Audio Exception: {e}")

    def stopAudio(self):
        with self.lock:
            self.logger.info(f"Muting source id={self.id}, name={self.name}")
            try:
                if self.player is not None:
                    self.player.disconnectAudio()
                self.playing = False
            except Exception as e:
                self.logger.error(f"Stop Audio Exception: {e}")

    def reportStatus(self):
        sourceDict = {}
        sourceDict["id"] = self.id
        if self.currently_playing is not None:
            url = "rtp://{}:{}".format(self.currently_playing.stream_ip, self.currently_playing.port)
            sourceDict["playing"] = url
        else:
            sourceDict["playing"] = "nothing"
        sourceDict["registered"] = "registered" if self.regResult else "unregistered"
        return sourceDict
