########################################################################################
#
#   Discovery.py
#
#   Discover IP Speakers using zeroconf python module
#
#   zeroconf - Multicast DNS Service Discovery for Python
#
########################################################################################

from zeroconf import ServiceBrowser, Zeroconf
import socket
import time
import json

class Discovery:
    def __init__(self, serv_type="_https._tcp.local."):
        self.serv_type = serv_type
        self.zeroconf = None
        self.running = False
        #self.logger = logging.getLogger('ipspeaker')
        self.run()

    def run(self):
        self.zeroconf = Zeroconf()
        listener = MyListener()
        ServiceBrowser(self.zeroconf, self.serv_type, listener)
        self.running = True
        try:
            while self.running:
                time.sleep(0.01)
        except KeyboardInterrupt:
            pass
        print("Leave run...")
        self.zeroconf.close()

    def __del__(self):
        self.zeroconf.close()

class MyListener:
    def __init__(self):
        self.discoverDict = {"discovery": list()}
        self.update_json()

    def remove_service(self, zeroconf, type, name):
        print("Service %s removed" % (name,))
        result = name.split("|")
        for elem in self.discoverDict["discovery"]:
            if elem["mac"] == str(result[1]):
                self.discoverDict["discovery"].remove(elem)
                self.update_json()
                print(self.discoverDict)
                break

    def add_service(self, zeroconf, type, name):
        info = zeroconf.get_service_info(type, name)
        host_ip = socket.inet_ntoa(info.addresses[0])
        print("Service %s added, service info: %s" % (name, info))
        print("Host: {}".format(host_ip))
        discover_host = dict()
        result = name.split("|")
        discover_host["name"] = result[0]
        discover_host["mac"] = result[1]
        discover_host["ip"] = host_ip
        #remove byte characters
        properties = { k.decode('utf-8') : v.decode('utf-8') for k, v in info.properties.items()}
        #discover_host["info"] = str(info.properties)
        discover_host["info"] = properties
        self.discoverDict["discovery"].append(discover_host)
        self.update_json()
        print(self.discoverDict)

    def update_json(self):
        f = open("/usr/local/www/current/app/discovery.json", 'w')
        f.write(json.dumps(self.discoverDict))
        f.close()

if __name__=="__main__":
    Discovery()