//
// Created by to on 17/02/21.
//

#ifndef BACO_INPUTBLOCK_H
#define BACO_INPUTBLOCK_H

#include <baco/Logger.h>

#include <baco/audio/FrameSink.h>
#include <baco/audio/FrameFormat.h>
#include <baco/audio/AudioResampler.h>
#include <baco/audio/AlsaFrameSource.h>
#include <baco/audio/AudioFifo.h>
#include <baco/audio/AudioGain.h>

struct InputBlockConfig
{
    std::string deviceName;
    int     sampleRate{ 0 };
    int     bufferSize{ 0 };
    int     periodSize{ 0 };
    bool    forceBufferConfig{ false };
    int     dispatchSize{ 0 };
    int     fifoDelay{ 0 }; ///< delay in frames

};

struct InputBlockStats
{
    uint64_t totalSamplesIn{0};
    int64_t  totalComp{0};
    int fifoLevel{0};
    int fifoLevelMax{INT_MIN};
    unsigned long tLevelMax{0};
    int fifoLevelMin{INT_MAX};
    unsigned long tLevelMin{0};
    int xrunIn{0};
    unsigned long tFirstXrun{0};
    int resamplerDelay;
};


class InputBlock
{
public:
    InputBlock(InputBlockConfig const& cfg, baco::FrameFormat const& outFmt);

    ~InputBlock();

    void attachSink( baco::FrameSinkPtr const& sink );

    baco::ReturnCode readFrame();

    /**
     *
     * @param clearLimits set to true to clear min and max stats for delay
     * @return
     */
    [[nodiscard]] InputBlockStats getStats()
    {
        if ( myResampler!=nullptr )  {
            myStats.resamplerDelay  = myResampler->getDelay(myInFmt.sampleRate());
        }
        return myStats;
    }

    void clearStats()
    {
        myLevelAcc              = 0;
        myLevelCount            = 0;
        myStats.xrunIn          = 0;
        myStats.fifoLevelMin    = INT_MAX;
        myStats.fifoLevelMax    = INT_MIN;
        myStats.tFirstXrun      = 0;
        myStats.tLevelMin       = 0;
        myStats.tLevelMax       = 0;
    }

    /**
     * call at every roughly 1s
     * @param numFrames Location where the decided compensation will be saved.
     *              May be omitted (nullptr)
     */
    int calculateCompensation();

    void applyCompensation( int comp );



    void setAvgComp( double comp )
    {
        log_information(logger,Poco::format("average compensation set to %.1f samples/s",comp));
        myResampler->setBackgroundCompensation(comp!=0,comp);
    }

    void setGain( double gain , int durationMs , int delayMs );

    /* HACKS!! BEWARE!! */
    void clearFifo()
    {
        myFifo->clear();
    }

private:

    Poco::Logger&               logger;
    InputBlockConfig            myConfig{};
    baco::FrameFormat           myInFmt;
    baco::AlsaFrameSourcePtr    myAlsaInput;
    baco::AudioGainPtr          myGain;
    baco::AudioResamplerPtr     myResampler;
    baco::AudioFifoPtr          myFifo;

    InputBlockStats             myStats{};
    double                      myLevelAcc{0};
    int                         myLevelCount{0};
};


#endif //BACO_INPUTBLOCK_H
